@@ -8,7 +8,7 @@ from bokeh.io import output_file, output_notebook
 from .basic import compute, plot, render
 from .correlation import compute_correlation, plot_correlation, render_correlation
 from .missing import compute_missing, plot_missing, render_missing
-from .utils import is_notebook, rand_str
+from .utils import is_notebook
 
 __all__ = [
     "plot_correlation",
@@ -24,5 +24,3 @@ __all__ = [
 
 if is_notebook():
     output_notebook(hide_banner=True)
-else:
-    output_file(filename=tempfile.gettempdir() + "/" + rand_str() + ".html")
