@@ -305,25 +305,19 @@ def compute_missing(
 
     Parameters
     ----------
-    pd_data_frame: pd.DataFrame
+    df
         the pandas data_frame for which plots are calculated for each column
-    x_name: str, optional
+    x
         a valid column name of the data frame
-    y_name: str, optional
+    y
         a valid column name of the data frame
-    ncols: int, optional
+    ncols
         The number of columns in the figure
-    bins: int
+    bins
         The number of rows in the figure
-    ndist_sample: int
+    ndist_sample
         The number of sample points
 
-    Returns
-    ----------
-    An object of figure or
-        An object of figure and
-        An intermediate representation for the plots of different columns in the data_frame.
-
     Examples
     ----------
     >>> from dataprep.eda.missing.computation import plot_missing
@@ -331,17 +325,6 @@ def compute_missing(
     >>> df = pd.read_csv("suicide-rate.csv")
     >>> plot_missing(df, "HDI_for_year")
     >>> plot_missing(df, "HDI_for_year", "population")
-
-    Notes
-    ----------
-    match (x_name, y_name)
-        case (Some, Some) => histogram for numerical column,
-        bars for categorical column, qq-plot, box-plot, jitter plot,
-        CDF, PDF
-        case (Some, None) => histogram for numerical column and
-        bars for categorical column
-        case (None, None) => heatmap
-        otherwise => error
     """
 
     df = to_dask(df)
